"""Tests for 'json_pbp.py'"""

import pandas as pd

from hockey_scraper.nhl.pbp import json_pbp


def test_get_pbp():
    """Tests to see we get something when scraping. We want it to return a dictionary"""
    assert isinstance(json_pbp.get_pbp("2016020001"), dict)
    assert isinstance(json_pbp.get_pbp("2008020768"), dict)


def test_get_teams():
    """Tests how extracting home and away teams from json"""
    assert json_pbp.get_teams(json_pbp.get_pbp("2014020001")) == {"Home": 'TOR', "Away": 'MTL'}


def test_parse_json():
    """ Tests how the pbp for one game is stored
        1. We want it to return a pandas DataFrame
        2. Checks to see if the proper game scraped is the right amount of events
        3. Checks if the right columns are included
    """
    scraped_game = json_pbp.scrape_game("2016020001")
    pbp_columns = ['period', 'event', 'seconds_elapsed', 'p1_name', 'p1_ID', 'p2_name', 'p2_ID', 'p3_name', 'p3_ID',
                   'xC', 'yC']

    assert isinstance(scraped_game, pd.DataFrame)
    assert scraped_game.shape[0] == 349
    assert list(scraped_game.columns) == pbp_columns


def test_parse_event():
    """ Test to see that it parses an event correctly"""

    event = {
        "players": [{
          "player": {
            "id": 8468575,
            "fullName": "Dominic Moore",
            "link": "/api/v1/people/8468575"
          },
          "playerType": "Scorer",
          "seasonTotal": 3
        }, {
          "player": {
            "id": 8470619,
            "fullName": "Brian Boyle",
            "link": "/api/v1/people/8470619"
          },
          "playerType": "Assist",
          "seasonTotal": 4
        }, {
          "player": {
            "id": 8474151,
            "fullName": "Ryan McDonagh",
            "link": "/api/v1/people/8474151"
          },
          "playerType": "Assist",
          "seasonTotal": 10
        }, {
          "player": {
            "id": 8474682,
            "fullName": "Dustin Tokarski",
            "link": "/api/v1/people/8474682"
          },
          "playerType": "Goalie"
        } ],
        "result": {
          "event": "Goal",
          "eventCode": "NYR294",
          "eventTypeId": "GOAL",
          "description": "Dominic Moore (3) Snap Shot, assists: Brian Boyle (4), Ryan McDonagh (10)",
          "secondaryType": "Snap Shot",
          "strength": {
            "code": "EVEN",
            "name": "Even"
          },
          "gameWinningGoal": True,
          "emptyNet": False
        },
        "about": {
          "eventIdx": 180,
          "eventId": 294,
          "period": 2,
          "periodType": "REGULAR",
          "ordinalNum": "2nd",
          "periodTime": "18:07",
          "periodTimeRemaining": "01:53",
          "dateTime": "2014-05-30T01:36:12Z",
          "goals": {
            "away": 0,
            "home": 1
          }
        },
        "coordinates": {
          "x": 77.0,
          "y": 5.0
        },
        "team": {
          "id": 3,
          "name": "New York Rangers",
          "link": "/api/v1/teams/3",
          "triCode": "NYR"
        }
    }

    parsed_event = {
        'event_id': 180, 'period': 2, 'event': 'GOAL', 'seconds_elapsed': 1087.0, 'p1_name': 'DOMINIC MOORE',
        'p1_ID': 8468575, 'p2_name': 'BRIAN BOYLE', 'p2_ID': 8470619, 'p3_name': 'RYAN MCDONAGH', 'p3_ID': 8474151,
        'xC': 77.0, 'yC': 5.0
    }

    assert json_pbp.parse_event(event) == parsed_event



""" Tests for 'html_pbp.py' """

import pandas as pd
import pytest

from hockey_scraper.nhl.pbp import html_pbp


# TODO: Fill out the rest of the test here and in the file (the important ones there)


@pytest.fixture
def game_id():
    return "2017020516"


@pytest.fixture
def cleaned_html(game_id):
    return html_pbp.clean_html_pbp(html_pbp.get_pbp(game_id))


@pytest.fixture
def pbp_cols():
    return ['Period', 'Event', 'Description', 'Time_Elapsed', 'Seconds_Elapsed', 'Strength', 'Ev_Zone', 'Type',
            'Ev_Team', 'Home_Zone', 'Away_Team', 'Home_Team', 'p1_name', 'p1_ID', 'p2_name', 'p2_ID', 'p3_name',
            'p3_ID', 'awayPlayer1', 'awayPlayer1_id', 'awayPlayer2', 'awayPlayer2_id', 'awayPlayer3', 'awayPlayer3_id',
            'awayPlayer4', 'awayPlayer4_id', 'awayPlayer5', 'awayPlayer5_id', 'awayPlayer6', 'awayPlayer6_id',
            'homePlayer1', 'homePlayer1_id', 'homePlayer2', 'homePlayer2_id', 'homePlayer3', 'homePlayer3_id',
            'homePlayer4', 'homePlayer4_id', 'homePlayer5', 'homePlayer5_id', 'homePlayer6', 'homePlayer6_id',
            'Away_Goalie', 'Away_Goalie_Id', 'Home_Goalie', 'Home_Goalie_Id', 'Away_Players', 'Home_Players',
            'Away_Score', 'Home_Score'
            ]


@pytest.fixture
def event():
    return ['112', '1', 'EV', '15:59', 'PENL', 'TOR #25 VAN RIEMSDYK\xa0Slashing(2 min), Off. Zone Drawn By: CAR #49 RASK',
            [['VICTOR RASK', '49', 'C'], ['JEFF SKINNER', '53', 'C'], ['TEUVO TERAVAINEN', '86', 'L'],
             ['NOAH HANIFIN', '5', 'D'], ['BRETT PESCE', '22', 'D'], ['SCOTT DARLING', '33', 'G']],
            [['MITCHELL MARNER', '16', 'C'], ['TYLER BOZAK', '42', 'C'], ['JAMES VAN RIEMSDYK', '25', 'L'],
             ['ROMAN POLAK', '46', 'D'], ['JAKE GARDINER', '51', 'D'], ['FREDERIK ANDERSEN', '31', 'G']]
            ]

@pytest.fixture
def players():
    return {'Home':
                {'RON HAINSEY': {'id': 8468493, 'number': '2', 'last_name': 'HAINSEY'},
                 'CONNOR CARRICK': {'id': 8476941, 'number': '8', 'last_name': 'CARRICK'},
                 'ZACH HYMAN': {'id': 8475786, 'number': '11', 'last_name': 'HYMAN'},
                 'PATRICK MARLEAU': {'id': 8466139, 'number': '12', 'last_name': 'MARLEAU'},
                 'MATT MARTIN': {'id': 8474709, 'number': '15', 'last_name': 'MARTIN'},
                 'MITCHELL MARNER': {'id': 8478483, 'number': '16', 'last_name': 'MARNER'},
                 'DOMINIC MOORE': {'id': 8468575, 'number': '20', 'last_name': 'MOORE'},
                 'KASPERI KAPANEN': {'id': 8477953, 'number': '24', 'last_name': 'KAPANEN'},
                 'JAMES VAN RIEMSDYK': {'id': 8474037, 'number': '25', 'last_name': 'VAN RIEMSDYK'},
                 'CONNOR BROWN': {'id': 8477015, 'number': '28', 'last_name': 'BROWN'},
                 'WILLIAM NYLANDER': {'id': 8477939, 'number': '29', 'last_name': 'NYLANDER'},
                 'TYLER BOZAK': {'id': 8475098, 'number': '42', 'last_name': 'BOZAK'},
                 'NAZEM KADRI': {'id': 8475172, 'number': '43', 'last_name': 'KADRI'},
                 'MORGAN RIELLY': {'id': 8476853, 'number': '44', 'last_name': 'RIELLY'},
                 'ROMAN POLAK': {'id': 8471392, 'number': '46', 'last_name': 'POLAK'},
                 'LEO KOMAROV': {'id': 8473463, 'number': '47', 'last_name': 'KOMAROV'},
                 'JAKE GARDINER': {'id': 8474581, 'number': '51', 'last_name': 'GARDINER'},
                 'ANDREAS BORGMAN': {'id': 8480158, 'number': '55', 'last_name': 'BORGMAN'},
                 'FREDERIK ANDERSEN': {'id': 8475883, 'number': '31', 'last_name': 'ANDERSEN'},
                 'JOSH LEIVO': {'id': 8476410, 'number': '32', 'last_name': 'LEIVO'},
                 'AUSTON MATTHEWS': {'id': 8479318, 'number': '34', 'last_name': 'MATTHEWS'},
                 'MARTIN MARINCIN': {'id': 8475716, 'number': '52', 'last_name': 'MARINCIN'}
                 },
            'Away':
                {'HAYDN FLEURY': {'id': 8477938, 'number': '4', 'last_name': 'FLEURY'},
                 'NOAH HANIFIN': {'id': 8478396, 'number': '5', 'last_name': 'HANIFIN'},
                 'DEREK RYAN': {'id': 8478585, 'number': '7', 'last_name': 'RYAN'},
                 'JORDAN STAAL': {'id': 8473533, 'number': '11', 'last_name': 'STAAL'},
                 'JUSTIN WILLIAMS': {'id': 8468508, 'number': '14', 'last_name': 'WILLIAMS'},
                 'MARCUS KRUGER': {'id': 8475323, 'number': '16', 'last_name': 'KRUGER'},
                 'JOSH JOORIS': {'id': 8477591, 'number': '19', 'last_name': 'JOORIS'},
                 'SEBASTIAN AHO': {'id': 8478427, 'number': '20', 'last_name': 'AHO'},
                 'BRETT PESCE': {'id': 8477488, 'number': '22', 'last_name': 'PESCE'},
                 'BROCK MCGINN': {'id': 8476934, 'number': '23', 'last_name': 'MCGINN'},
                 'JUSTIN FAULK': {'id': 8475753, 'number': '27', 'last_name': 'FAULK'},
                 'ELIAS LINDHOLM': {'id': 8477496, 'number': '28', 'last_name': 'LINDHOLM'},
                 'JOAKIM NORDSTROM': {'id': 8475807, 'number': '42', 'last_name': 'NORDSTROM'},
                 'VICTOR RASK': {'id': 8476437, 'number': '49', 'last_name': 'RASK'},
                 'JEFF SKINNER': {'id': 8475784, 'number': '53', 'last_name': 'SKINNER'},
                 'TREVOR VAN RIEMSDYK': {'id': 8477845, 'number': '57', 'last_name': 'VAN RIEMSDYK'},
                 'JACCOB SLAVIN': {'id': 8476958, 'number': '74', 'last_name': 'SLAVIN'},
                 'TEUVO TERAVAINEN': {'id': 8476882, 'number': '86', 'last_name': 'TERAVAINEN'},
                 'SCOTT DARLING': {'id': 8474152, 'number': '33', 'last_name': 'DARLING'},
                 'KLAS DAHLBECK': {'id': 8476403, 'number': '6', 'last_name': 'DAHLBECK'},
                 'PHILLIP DI GIUSEPPE': {'id': 8476858, 'number': '34', 'last_name': 'DI GIUSEPPE'}
                 }
            }


@pytest.fixture
def teams():
    return {'Home': 'TOR', 'Away': 'CAR'}


@pytest.fixture
def current_score():
    return {'Home': 4, 'Away': 1}


def test_parse_event(event, players, teams, current_score):
    """ Checks that it parses an event correctly """
    parsed_event = {
        'Description': 'TOR #25 VAN RIEMSDYK\xa0Slashing(2 min), Off. Zone Drawn By: CAR #49 RASK',
        'Event': 'PENL', 'Period': 1, 'Time_Elapsed': '15:59', 'Seconds_Elapsed': 959.0, 'Ev_Team': 'TOR',
        'Home_Score': 4, 'Away_Score': 1, 'score_diff': 3, 'homePlayer1': 'MITCHELL MARNER', 'homePlayer1_id': 8478483,
        'homePlayer2': 'TYLER BOZAK', 'homePlayer2_id': 8475098, 'homePlayer3': 'JAMES VAN RIEMSDYK',
        'homePlayer3_id': 8474037, 'homePlayer4': 'ROMAN POLAK', 'homePlayer4_id': 8471392,
        'homePlayer5': 'JAKE GARDINER', 'homePlayer5_id': 8474581, 'homePlayer6': 'FREDERIK ANDERSEN',
        'homePlayer6_id': 8475883, 'awayPlayer1': 'VICTOR RASK', 'awayPlayer1_id': 8476437,
        'awayPlayer2': 'JEFF SKINNER', 'awayPlayer2_id': 8475784, 'awayPlayer3': 'TEUVO TERAVAINEN',
        'awayPlayer3_id': 8476882, 'awayPlayer4': 'NOAH HANIFIN', 'awayPlayer4_id': 8478396,
        'awayPlayer5': 'BRETT PESCE', 'awayPlayer5_id': 8477488, 'awayPlayer6': 'SCOTT DARLING',
        'awayPlayer6_id': 8474152, 'Away_Goalie': 'SCOTT DARLING', 'Away_Goalie_Id': 8474152,
        'Home_Goalie': 'FREDERIK ANDERSEN', 'Home_Goalie_Id': 8475883, 'Away_Players': 6, 'Home_Players': 6,
        'Strength': '5x5', 'Type': 'Slashing(2 min)', 'Ev_Zone': 'Off', 'Home_Zone': 'Off',
        'p1_name': 'JAMES VAN RIEMSDYK', 'p1_ID': 8474037, 'p2_name': 'VICTOR RASK', 'p2_ID': 8476437
    }

    assert parsed_event == html_pbp.parse_event(event, players, teams['Home'], current_score)


def test_parse_html(pbp_cols, players, teams, cleaned_html):
    """ Check that it parsed the entirety of the html correctly"""
    game_df = html_pbp.parse_html(cleaned_html, players, teams)

    assert isinstance(game_df, pd.DataFrame)
    assert game_df.shape[0] == 331
    assert list(game_df.columns) == pbp_cols


def test_get_pbp():
    """ Test getting the html pbp"""
    pass


def test_get_soup():
    """ Test 'soupifying' the html doc """
    pass


def test_strip_html_pbp():
    """ String the html tags and such and make a list of lists for all plays"""
    pass


def test_clean_html_pbp():
    """ Get rid of html and format the data """
    pass



from .nwhl import scrape_functions as nwhl
from .nhl.live_scrape import ScrapeLiveGames, LiveGame
from .nhl.scrape_functions import scrape_games, scrape_date_range, scrape_seasons
from .utils import shared



"""Test for 'playing_roster.py'"""

import pytest

from hockey_scraper.nhl import playing_roster


@pytest.fixture
def scraped_roster():
    return playing_roster.scrape_roster("2016020475")


def test_fix_name():
    """ Tests to see that it takes the assistant captain and regular captain ('(A)' and '(C)') out of player's names."""
    assert playing_roster.fix_name(['27', 'D', 'RYAN MCDONAGH  (C)', False]) == ['27', 'D', 'RYAN MCDONAGH', False]
    assert playing_roster.fix_name(['5', 'D', 'DAN GIRARDI  (A)', False]) == ['5', 'D', 'DAN GIRARDI', False]
    assert playing_roster.fix_name(['13', 'R', 'KEVIN HAYES', False]) == ['13', 'R', 'KEVIN HAYES', False]


def test_get_players(scraped_roster):
    """ Tests if it get the correct players for both teams in the correct format. """
    home_roster = [
        ['5', 'D', 'DAN GIRARDI', False],
        ['8', 'D', 'KEVIN KLEIN', False],
        ['10', 'C', 'J.T. MILLER', False],
        ['12', 'L', 'MATT PUEMPEL', False],
        ['13', 'R', 'KEVIN HAYES', False],
        ['18', 'D', 'MARC STAAL', False],
        ['19', 'R', 'JESPER FAST', False],
        ['20', 'C', 'CHRIS KREIDER', False],
        ['21', 'C', 'DEREK STEPAN', False],
        ['22', 'D', 'NICK HOLDEN', False],
        ['24', 'C', 'OSCAR LINDBERG', False],
        ['26', 'L', 'JIMMY VESEY', False],
        ['27', 'D', 'RYAN MCDONAGH', False],
        ['36', 'C', 'MATS ZUCCARELLO', False],
        ['40', 'R', 'MICHAEL GRABNER', False],
        ['46', 'L', 'MAREK HRIVIK', False],
        ['61', 'L', 'RICK NASH', False],
        ['76', 'D', 'BRADY SKJEI', False],
        ['30', 'G', 'HENRIK LUNDQVIST', False],
        ['32', 'G', 'ANTTI RAANTA', False],
        ['4', 'D', 'ADAM CLENDENING', True],
        ['73', 'C', 'BRANDON PIRRI', True]
    ]
    away_roster = [
        ['2', 'D', 'JOHN MOORE', False],
        ['6', 'D', 'ANDY GREENE', False],
        ['7', 'D', 'JON MERRILL', False],
        ['8', 'R', 'BEAU BENNETT', False],
        ['9', 'L', 'TAYLOR HALL', False],
        ['11', 'R', 'PA PARENTEAU', False],
        ['12', 'D', 'BEN LOVEJOY', False],
        ['13', 'L', 'MICHAEL CAMMALLERI', False],
        ['14', 'C', 'ADAM HENRIQUE', False],
        ['19', 'C', 'TRAVIS ZAJAC', False],
        ['21', 'C', 'KYLE PALMIERI', False],
        ['22', 'D', 'KYLE QUINCEY', False],
        ['28', 'D', 'DAMON SEVERSON', False],
        ['36', 'R', 'NICK LAPPIN', False],
        ['37', 'C', 'PAVEL ZACHA', False],
        ['38', 'L', 'VERNON FIDDLER', False],
        ['44', 'L', 'MILES WOOD', False],
        ['51', 'C', 'SERGEY KALININ', False],
        ['1', 'G', 'KEITH KINKAID', False],
        ['35', 'G', 'CORY SCHNEIDER', False],
        ['16', 'C', 'JACOB JOSEFSON', True],
        ['20', 'L', 'LUKE GAZDIC', True],
        ['25', 'R', 'DEVANTE SMITH-PELLY', True]
    ]

    assert 'Home' in scraped_roster['players']
    assert 'Away' in scraped_roster['players']

    assert scraped_roster['players']['Home'] == home_roster
    assert scraped_roster['players']['Away'] == away_roster


def test_get_coaches(scraped_roster):
    """ Tests if it get the correct coaches for both teams in the correct format. """
    assert 'Home' in scraped_roster['head_coaches']
    assert 'Away' in scraped_roster['head_coaches']

    assert scraped_roster['head_coaches']['Home'] == 'ALAIN VIGNEAULT'
    assert scraped_roster['head_coaches']['Away'] == 'JOHN HYNES'


def test_scrape_roster(scraped_roster):
    """ Test scraping all the roster info """
    assert 'players' in scraped_roster
    assert 'head_coaches' in scraped_roster



""" Tests for 'espn_pbp.py' """

import pandas as pd
import pytest

from hockey_scraper.nhl.pbp import espn_pbp


@pytest.fixture
def game_date():
    return '2015-10-24'


@pytest.fixture
def teams():
    return {'Home': 'PHI', 'Away': 'NYR'}


@pytest.fixture
def game_response(game_date, teams):
    """ Page for that game"""
    return espn_pbp.get_espn_game(game_date, teams['Home'], teams['Away'])


@pytest.fixture
def date_response(game_date):
    """ Page that details all the games that day"""
    return espn_pbp.get_espn_date(game_date)


def test_get_teams(date_response):
    """ Check to make sure we get a list of both teams for every game that day"""

    # Games for that date
    date_games = [['ANA', 'MIN'], ['N.J', 'BUF'], ['TOR', 'MTL'], ['PHX', 'OTT'], ['NYR', 'PHI'], ['NYI', 'STL'],
                  ['PIT', 'NSH'], ['FLA', 'DAL'], ['T.B', 'CHI'], ['CBJ', 'COL'], ['DET', 'VAN'], ['CAR', 'S.J']]

    assert espn_pbp.get_teams(date_response) == date_games


def test_get_game_ids(date_response):
    """ Check to see that all the espn game id's for that day are correct"""
    game_ids = ['400814970', '400814971', '400814972', '400814973', '400814974', '400814975', '400814976',
                '400814977', '400814978', '400814979', '400814980', '400814981']

    assert espn_pbp.get_game_ids(date_response) == game_ids


def test_get_espn_game_id(game_date, teams):
    """ 
    Test to see for a given game (identified by the game_id and both teams) it return the correct game_id number
    for the Espn api.
    """
    assert espn_pbp.get_espn_game_id(game_date, teams['Home'], teams['Away']) == '400814974'


def test_get_espn_game(game_response):
    """ Test to see if we get anything back whn requesting a game. Should be a non-empty string"""
    assert type(game_response) == str
    assert len(game_response) > 0


def test_parse_event():
    """ Checks to see that it correctly parses a game event"""
    event = "-35~12~505~4:48~2~3506~5767~5833~Power Play Goal Scored by Derick Brassard(Slapshot 56 ft)assisted by " \
            "Kevin Hayes and Chris Kreider~0~2~2~0~702~13~801~901~2~3~4"
    parsed_event = {"xC": -35, "yC": 12, "time_elapsed": 288, 'period': '2', 'event': "GOAL"}

    assert espn_pbp.parse_event(event) == parsed_event


def test_parse_espn(game_response):
    """ Checks if the espn game response is parsed correctly. Specifically: 
        1. Is a DataFrame?
        2. Contains the correct amount of events?
        3. Contains the correct columns?
    """
    scraped_game = espn_pbp.parse_espn(game_response)
    espn_columns = ['period', 'time_elapsed', 'event', 'xC', 'yC']

    assert isinstance(scraped_game, pd.DataFrame)
    assert scraped_game.shape[0] == 379
    assert list(scraped_game.columns) == espn_columns



""" Tests for 'shared.py' """

import os
import shutil

import pytest

from hockey_scraper.utils import shared


@pytest.fixture
def file_info():
    return {
        "url": 'http://statsapi.web.nhl.com/api/v1/game/{}/feed/live'.format(2017020001),
        "name": str(2017020001),
        "type": "json_pbp",
        "season": 2017,
    }


def test_check_data_format():
    """ Test if it recognized the correct formats allowed"""
    # These both are fine
    shared.check_data_format("Csv")
    shared.check_data_format("pandaS")

    # Should raise an exception
    with pytest.raises(ValueError):
        shared.check_data_format("txt")


def test_check_valid_dates():
    """ Test if given valid date range"""
    shared.check_valid_dates("2017-10-01", "2018-01-05")

    with pytest.raises(ValueError):
        shared.check_valid_dates("2017-12-01", "2017-11-30")


def test_convert_to_seconds():
    """ Tests if it correctly converts minutes remaining to seconds elapsed"""
    assert shared.convert_to_seconds("8:33") == 513
    assert shared.convert_to_seconds("-16:0-") == "1200"


def test_get_season():
    """ Tests that this function returns the correct season for a given date"""
    assert shared.get_season("2017-10-01") == 2017
    assert shared.get_season("2016-06-01") == 2015


def test_scrape_page(file_info):
    """ Test scraping from the source is good"""
    file = shared.scrape_page(file_info['url'])

    assert type(file) == str
    assert len(file) > 0


def test_add_dir():
    """ Test if this function correctly tells if a directory exists on the machine"""

    # Check when it does exist (will always be good for this file)
    user_dir = os.path.dirname(os.path.realpath(__file__))
    shared.add_dir(user_dir)
    assert shared.docs_dir is not None

    # Checks when it doesn't exist
    user_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "hopefully_this_path_doesnt_exist")
    shared.add_dir(user_dir)
    assert shared.docs_dir is False


def test_get_file(file_info):
    """ Test getting the file...it's either scraped or loaded from a file """
    original_path = os.getcwd()

    # When there is either no directory specified or it doesn't exist
    file = shared.get_file(file_info)
    assert type(file) == str
    assert len(file) > 0
    assert original_path == os.getcwd()

    # When the directory exists
    # Here I just use the directory of this file to make things easy
    shared.add_dir(os.path.dirname(os.path.realpath(__file__)))
    file = shared.get_file(file_info)
    assert type(file) == str
    assert len(file) > 0
    assert original_path == os.getcwd()

    # Some cleanup....remove stuff created from the file directory and move back
    os.chdir(os.path.dirname(os.path.realpath(__file__)))
    shutil.rmtree("docs")
    shutil.rmtree("csvs")
    os.chdir(original_path)



""" Tests for 'html_shifts.py' """

import bs4
import pandas as pd
import pytest

from hockey_scraper.nhl.shifts import html_shifts


@pytest.fixture
def shift_cols():
    return ['Game_Id', 'Period', 'Team', 'Player', 'Player_Id', 'Start', 'End', 'Duration']


@pytest.fixture
def game_id():
    return '2009020001'

@pytest.fixture
def player_ids():
    return {
        'Home': {
                  'DENNIS WIDEMAN': {'id': 8469770, 'number': '6', 'last_name': 'WIDEMAN'},
                  'CHUCK KOBASEW': {'id': 8469467, 'number': '12', 'last_name': 'KOBASEW'},
                  'MARCO STURM': {'id': 8464979, 'number': '16', 'last_name': 'STURM'},
                  'MILAN LUCIC': {'id': 8473473, 'number': '17', 'last_name': 'LUCIC'},
                  'ANDREW FERENCE': {'id': 8466333, 'number': '21', 'last_name': 'FERENCE'},
                  'SHAWN THORNTON': {'id': 8465978, 'number': '22', 'last_name': 'THORNTON'},
                  'BLAKE WHEELER': {'id': 8471218, 'number': '26', 'last_name': 'WHEELER'},
                  'STEVE BEGIN': {'id': 8464994, 'number': '27', 'last_name': 'BEGIN'},
                  'MARK RECCHI': {'id': 8450725, 'number': '28', 'last_name': 'RECCHI'},
                  'ZDENO CHARA': {'id': 8465009, 'number': '33', 'last_name': 'CHARA'},
                  'PATRICE BERGERON': {'id': 8470638, 'number': '37', 'last_name': 'BERGERON'},
                  'MARK STUART': {'id': 8470614, 'number': '45', 'last_name': 'STUART'},
                  'DAVID KREJCI': {'id': 8471276, 'number': '46', 'last_name': 'KREJCI'},
                  'MATT HUNWICK': {'id': 8471436, 'number': '48', 'last_name': 'HUNWICK'},
                  'DEREK MORRIS': {'id': 8464966, 'number': '53', 'last_name': 'MORRIS'},
                  'BYRON BITZ': {'id': 8470700, 'number': '61', 'last_name': 'BITZ'},
                  'MICHAEL RYDER': {'id': 8467545, 'number': '73', 'last_name': 'RYDER'},
                  'MARC SAVARD': {'id': 8462118, 'number': '91', 'last_name': 'SAVARD'},
                  'TIM THOMAS': {'id': 8460703, 'number': '30', 'last_name': 'THOMAS'},
                  'JOHNNY BOYCHUK': {'id': 8470187, 'number': '55', 'last_name': 'BOYCHUK'}
                  },
        'Away': {
                  'BRIAN POTHIER': {'id': 8468427, 'number': '2', 'last_name': 'POTHIER'},
                  'TOM POTI': {'id': 8465012, 'number': '3', 'last_name': 'POTI'},
                  'JOHN ERSKINE': {'id': 8467365, 'number': '4', 'last_name': 'ERSKINE'},
                  'ALEX OVECHKIN': {'id': 8471214, 'number': '8', 'last_name': 'OVECHKIN'},
                  'BRENDAN MORRISON': {'id': 8459461, 'number': '9', 'last_name': 'MORRISON'},
                  'MATT BRADLEY': {'id': 8465059, 'number': '10', 'last_name': 'BRADLEY'},
                  'BOYD GORDON': {'id': 8470159, 'number': '15', 'last_name': 'GORDON'},
                  'CHRIS CLARK': {'id': 8460567, 'number': '17', 'last_name': 'CLARK'},
                  'NICKLAS BACKSTROM': {'id': 8473563, 'number': '19', 'last_name': 'BACKSTROM'},
                  'BROOKS LAICH': {'id': 8469639, 'number': '21', 'last_name': 'LAICH'},
                  'MIKE KNUBLE': {'id': 8458590, 'number': '22', 'last_name': 'KNUBLE'},
                  'MILAN JURCINA': {'id': 8469684, 'number': '23', 'last_name': 'JURCINA'},
                  'SHAONE MORRISONN': {'id': 8469472, 'number': '26', 'last_name': 'MORRISONN'},
                  'ALEXANDER SEMIN': {'id': 8470120, 'number': '28', 'last_name': 'SEMIN'},
                  'BOYD KANE': {'id': 8465028, 'number': '34', 'last_name': 'KANE'},
                  'DAVID STECKEL': {'id': 8469483, 'number': '39', 'last_name': 'STECKEL'},
                  'MIKE GREEN': {'id': 8471242, 'number': '52', 'last_name': 'GREEN'},
                  'QUINTIN LAING': {'id': 8466232, 'number': '53', 'last_name': 'LAING'},
                  'JOSE THEODORE': {'id': 8460535, 'number': '60', 'last_name': 'THEODORE'},
                  'JEFF SCHULTZ': {'id': 8471240, 'number': '55', 'last_name': 'SCHULTZ'},
                  'TYLER SLOAN': {'id': 8468846, 'number': '89', 'last_name': 'SLOAN'},
                  'MICHAEL NYLANDER': {'id': 8458573, 'number': '92', 'last_name': 'NYLANDER'}
        }
    }


@pytest.fixture
def shifts_html():
    home, away = html_shifts.get_shifts("2009020001")
    return {'home': home, 'away': away}


@pytest.fixture
def shifts_dfs(shifts_html, player_ids, game_id):
    home_df = html_shifts.parse_html(shifts_html['home'], player_ids, game_id)
    away_df = html_shifts.parse_html(shifts_html['away'], player_ids, game_id)

    return {'home': home_df, 'away': away_df}


def test_get_shifts(shifts_html):
    """ Test getting both shifts pages """
    assert type(shifts_html['home']) == str
    assert len(shifts_html['home']) > 0

    assert type(shifts_html['away']) == str
    assert len(shifts_html['away']) > 0


def test_get_soup(shifts_html):
    """ Test get soup -> Returns td tags for pbp and= list of both teams"""

    # Home
    td, teams = html_shifts.get_soup(shifts_html['home'])
    assert type(td) == bs4.element.ResultSet
    assert len(td) > 100   # If it's greater than 100 it's fine
    assert type(teams) == list
    assert len(teams) == 2

    # Away
    td, teams = html_shifts.get_soup(shifts_html['away'])
    assert type(td) == bs4.element.ResultSet
    assert len(td) > 100  # If it's greater than 100 it's fine
    assert type(teams) == list
    assert len(teams) == 2


def test_analyze_shifts(player_ids):
    """ Test analyzing a single shift. See if it parses it correctly"""
    # Get html (one td) and see if it spits out the correct info
    shift = ['30', '3', '17:05 / 2:55', '18:03 / 1:57', '00:58']
    name = "DENNIS WIDEMAN"
    team = "BOSTON BRUINS"
    home_team = "BOSTON BRUINS"
    parsed_shift = {'Player': 'DENNIS WIDEMAN', 'Period': '3', 'Team': 'BOS', 'Start': 1025.0, 'Duration': 58.0,
                    'End': 1083.0, 'Player_Id': 8469770}

    assert parsed_shift == html_shifts.analyze_shifts(shift, name, team, home_team, player_ids)


def test_parse_html(shifts_dfs, shift_cols):
    """ Should return a DataFrame of all the shifts for a given team 
        Here we make sure it works for both the home and away teams
    """
    # Check they are both DataFrames
    assert isinstance(shifts_dfs['home'], pd.DataFrame)
    assert isinstance(shifts_dfs['away'], pd.DataFrame)

    # Check correct amount of rows for each
    assert shifts_dfs['home'].shape[0] == 360
    assert shifts_dfs['away'].shape[0] == 375

    # Correct columns for each
    assert list(shifts_dfs['home'].columns).sort() == shift_cols.sort()
    assert list(shifts_dfs['away'].columns).sort() == shift_cols.sort()


def test_scrape_game(game_id, player_ids, shifts_dfs, shift_cols):
    """ Should return DataFrame with both home and away shifts """
    # Return Df
    # Correct amount of line  -- Confirm equals to home + away
    # Correct columns
    game_df = html_shifts.scrape_game(game_id, player_ids)

    # Check if it's the right datatype
    assert isinstance(game_df, pd.DataFrame)

    # Same number of rows as correct value and addition of home and aways dfs
    assert game_df.shape[0] == (shifts_dfs['home'].shape[0] + shifts_dfs['away'].shape[0]) == 735

    # Correct columns
    assert list(shifts_dfs['home'].columns).sort() == shift_cols.sort()



"""Tests for 'json_schedule.py'"""
import datetime

from hockey_scraper.nhl import json_schedule


def test_get_schedule():
    """Tests to see we get something when scraping. We want it to return a dictionary"""
    assert isinstance(json_schedule.get_schedule("2018-03-28", "2018-07-01"), dict)
    assert isinstance(json_schedule.get_schedule("2017-09-01", "2018-09-15"), dict)


def test_scrape_schedule():
    """Test to see if successfully get the correct number of games between two dates"""
    assert len(json_schedule.scrape_schedule("2017-08-01", "2017-09-01")) == 0
    assert len(json_schedule.scrape_schedule("2017-09-01", "2017-11-15")) == 277
    assert len(json_schedule.scrape_schedule("2017-09-01", "2017-11-15", preseason="True")) == 385


def test_get_dates():
    """Test to see that it returns the correct dates for given game id's"""
    #print(json_schedule.get_dates([2015010002])[0])
    #print(json_schedule.get_dates([2017020275])[0])
    #print(json_schedule.get_dates([2014030416])[0])
    assert json_schedule.get_dates([2015010002])[0] == {'game_id': 2015010002, 'date': '2015-09-20',
                                                        'start_time': datetime.datetime(2015, 9, 20, 20, 30),
                                                        'home_team': 'NSH', 'away_team': 'FLA', 'status': 'Final'}
    assert json_schedule.get_dates([2017020275])[0] == {'game_id': 2017020275, 'date': '2017-11-15',
                                                        'start_time': datetime.datetime(2017, 11, 16, 0, 30),
                                                        'home_team': 'DET', 'away_team': 'CGY', 'status': 'Final'}
    assert json_schedule.get_dates([2014030416])[0] == {'game_id': 2014030416, 'date': '2015-06-15',
                                                        'start_time': datetime.datetime(2015, 6, 16, 0, 0),
                                                        'home_team': 'CHI', 'away_team': 'T.B', 'status': 'Final'}



"""Tests for 'json_shifts.py'"""

import pandas as pd

from hockey_scraper.nhl.shifts import json_shifts


def test_get_shifts():
    """Tests to see we get something when scraping. We want it to return a dictionary"""
    assert isinstance(json_shifts.get_shifts("2016020001"), dict)
    assert isinstance(json_shifts.get_shifts("2008020768"), dict)


def test_scrape_shifts():
    """ Tests scraping the json shifts. 
        1. We either want a pandas df or None.
        2. Checks to see if the proper game scraped is the right amount of shifts
        3. Checks if right columns are included
    """
    scraped_shifts = json_shifts.scrape_game("2016020001")

    assert isinstance(scraped_shifts, pd.DataFrame)

    assert scraped_shifts.shape[0] == 850

    shift_columns = ['Game_Id', 'Period', 'Team', 'Player', 'Player_Id', 'Start', 'End', 'Duration']
    assert list(scraped_shifts.columns) == shift_columns


def test_parse_shift():
    """ Test to see that it parses a shift correctly"""
    shift = {
        "detailCode": 0, "duration": "00:46", "endTime": "04:12", "eventDescription": None, "eventDetails": None,
        "eventNumber": 327, "firstName": "Leo", "gameId": 2016020001, "hexValue": "#00205B", "lastName": "Komarov",
        "period": 2, "playerId": 8473463, "shiftNumber": 12, "startTime": "03:26", "teamAbbrev": "TOR", "teamId": 10,
        "teamName": "Toronto Maple Leafs", "typeCode": 517
    }

    parsed_shift = {'Player': 'LEO KOMAROV', 'Player_Id': 8473463, 'Period': 2, 'Team': 'TOR', 'Start': 206.0,
                    'End': 252.0, 'Duration': 46.0}

    assert json_shifts.parse_shift(shift) == parsed_shift



""" Tests for 'game_scraper.py' """

import pandas as pd
import pytest

from hockey_scraper.nhl import game_scraper, playing_roster
from hockey_scraper.nhl.pbp import json_pbp


@pytest.fixture
def players():
    return {'Home':
               {'NOAH HANIFIN': {'id': 8478396, 'number': '5', 'last_name': 'HANIFIN'},
                'KLAS DAHLBECK': {'id': 8476403, 'number': '6', 'last_name': 'DAHLBECK'},
                'DEREK RYAN': {'id': 8478585, 'number': '7', 'last_name': 'RYAN'},
                'JORDAN STAAL': {'id': 8473533, 'number': '11', 'last_name': 'STAAL'},
                'JUSTIN WILLIAMS': {'id': 8468508, 'number': '14', 'last_name': 'WILLIAMS'},
                'SEBASTIAN AHO': {'id': 8478427, 'number': '20', 'last_name': 'AHO'},
                'LEE STEMPNIAK': {'id': 8470740, 'number': '21', 'last_name': 'STEMPNIAK'},
                'BRETT PESCE': {'id': 8477488, 'number': '22', 'last_name': 'PESCE'},
                'BROCK MCGINN': {'id': 8476934, 'number': '23', 'last_name': 'MCGINN'},
                'JUSTIN FAULK': {'id': 8475753, 'number': '27', 'last_name': 'FAULK'},
                'ELIAS LINDHOLM': {'id': 8477496, 'number': '28', 'last_name': 'LINDHOLM'},
                'PHILLIP DI GIUSEPPE': {'id': 8476858, 'number': '34', 'last_name': 'DI GIUSEPPE'},
                'JOAKIM NORDSTROM': {'id': 8475807, 'number': '42', 'last_name': 'NORDSTROM'},
                'VICTOR RASK': {'id': 8476437, 'number': '49', 'last_name': 'RASK'},
                'JEFF SKINNER': {'id': 8475784, 'number': '53', 'last_name': 'SKINNER'},
                'TREVOR VAN RIEMSDYK': {'id': 8477845, 'number': '57', 'last_name': 'VAN RIEMSDYK'},
                'JACCOB SLAVIN': {'id': 8476958, 'number': '74', 'last_name': 'SLAVIN'},
                'TEUVO TERAVAINEN': {'id': 8476882, 'number': '86', 'last_name': 'TERAVAINEN'},
                'CAM WARD': {'id': 8470320, 'number': '30', 'last_name': 'WARD'},
                'HAYDN FLEURY': {'id': 8477938, 'number': '4', 'last_name': 'FLEURY'},
                'PATRICK BROWN': {'id': 8477887, 'number': '36', 'last_name': 'BROWN'}
                },
            'Away':
                {'NICK LEDDY': {'id': 8475181, 'number': '2', 'last_name': 'LEDDY'},
                 'RYAN PULOCK': {'id': 8477506, 'number': '6', 'last_name': 'PULOCK'},
                 'JORDAN EBERLE': {'id': 8474586, 'number': '7', 'last_name': 'EBERLE'},
                 'JOSH BAILEY': {'id': 8474573, 'number': '12', 'last_name': 'BAILEY'},
                 'MATHEW BARZAL': {'id': 8478445, 'number': '13', 'last_name': 'BARZAL'},
                 'THOMAS HICKEY': {'id': 8474066, 'number': '14', 'last_name': 'HICKEY'},
                 'CAL CLUTTERBUCK': {'id': 8473504, 'number': '15', 'last_name': 'CLUTTERBUCK'},
                 'ANDREW LADD': {'id': 8471217, 'number': '16', 'last_name': 'LADD'},
                 'ANDERS LEE': {'id': 8475314, 'number': '27', 'last_name': 'LEE'},
                 'SEBASTIAN AHO': {'id': 8480222, 'number': '28', 'last_name': 'AHO'},
                 'BROCK NELSON': {'id': 8475754, 'number': '29', 'last_name': 'NELSON'},
                 'ADAM PELECH': {'id': 8476917, 'number': '50', 'last_name': 'PELECH'},
                 'ROSS JOHNSTON': {'id': 8477527, 'number': '52', 'last_name': 'JOHNSTON'},
                 'CASEY CIZIKAS': {'id': 8475231, 'number': '53', 'last_name': 'CIZIKAS'},
                 'JOHNNY BOYCHUK': {'id': 8470187, 'number': '55', 'last_name': 'BOYCHUK'},
                 'TANNER FRITZ': {'id': 8479206, 'number': '56', 'last_name': 'FRITZ'},
                 'ANTHONY BEAUVILLIER': {'id': 8478463, 'number': '72', 'last_name': 'BEAUVILLIER'},
                 'JOHN TAVARES': {'id': 8475166, 'number': '91', 'last_name': 'TAVARES'},
                 'THOMAS GREISS': {'id': 8471306, 'number': '1', 'last_name': 'GREISS'},
                 'DENNIS SEIDENBERG': {'id': 8469619, 'number': '4', 'last_name': 'SEIDENBERG'},
                 'ALAN QUINE': {'id': 8476409, 'number': '10', 'last_name': 'QUINE'},
                 'JASON CHIMERA': {'id': 8466251, 'number': '25', 'last_name': 'CHIMERA'}
                 }
            }


@pytest.fixture
def pbp_columns():
    return ['Game_Id', 'Date', 'Period', 'Event', 'Description', 'Time_Elapsed', 'Seconds_Elapsed', 'Strength',
            'Ev_Zone', 'Type', 'Ev_Team', 'Home_Zone', 'Away_Team', 'Home_Team', 'p1_name', 'p1_ID', 'p2_name', 'p2_ID',
            'p3_name', 'p3_ID', 'awayPlayer1', 'awayPlayer1_id', 'awayPlayer2', 'awayPlayer2_id', 'awayPlayer3',
            'awayPlayer3_id', 'awayPlayer4', 'awayPlayer4_id', 'awayPlayer5', 'awayPlayer5_id', 'awayPlayer6',
            'awayPlayer6_id', 'homePlayer1', 'homePlayer1_id', 'homePlayer2', 'homePlayer2_id', 'homePlayer3',
            'homePlayer3_id', 'homePlayer4', 'homePlayer4_id', 'homePlayer5', 'homePlayer5_id', 'homePlayer6',
            'homePlayer6_id', 'Away_Players', 'Home_Players', 'Away_Score', 'Home_Score', 'Away_Goalie',
            'Away_Goalie_Id', 'Home_Goalie', 'Home_Goalie_Id', 'xC', 'yC', 'Home_Coach', 'Away_Coach'
            ]


@pytest.fixture
def shifts_columns():
    return ['Game_Id', 'Period', 'Team', 'Player', 'Player_Id', 'Start', 'End', 'Duration', 'Date']


def test_scrape_game(pbp_columns, shifts_columns):
    """ Tests if scrape pbp and shifts for game correctly with and without shifts.
        Check:
            1. Returns either a DataFrame or None (for shifts when False)
            2. The number of rows is correct
            3. The columns are correct
     """

    # 1. Try first without shifts
    pbp, shifts = game_scraper.scrape_game("2016020475", "2016-12-18", False)
    assert isinstance(pbp, pd.DataFrame)
    assert shifts is None
    assert pbp.shape[0] == 326
    assert list(pbp.columns) == pbp_columns

    # 2. Try with shifts
    pbp, shifts = game_scraper.scrape_game("2007020222", "2007-11-08", True)
    assert isinstance(pbp, pd.DataFrame)
    assert isinstance(shifts, pd.DataFrame)
    assert pbp.shape[0] == 248
    assert shifts.shape[0] == 726
    assert list(pbp.columns) == pbp_columns
    assert list(shifts.columns) == shifts_columns


def test_combine_players_lists(players):
    """ Check that it combines the list of players from the json pbp and the html roster correctly """
    game_id = "2017020891"
    json_players = game_scraper.get_players_json(json_pbp.get_pbp(game_id)['gameData']['players'])
    roster = playing_roster.scrape_roster(game_id)['players']

    assert players == game_scraper.combine_players_lists(json_players, roster, game_id)



import os
from setuptools import setup, find_packages


def read():
    return open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

setup(
    name='hockey_scraper',
    version='1.33',
    description="""This package is designed to allow one to scrape the raw data for both the National Hockey League
                   (NHL) and the National Women's Hockey League (NWHL) off of their respective API and websites.""",
    long_description=read(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        'Intended Audience :: Science/Research',
        "License :: OSI Approved :: MIT License",
        'Programming Language :: Python :: 3',
        "Programming Language :: Python :: 2",
    ],
    keywords='NHL',
    url='https://github.com/HarryShomer/Hockey-Scraper',
    author='Harry Shomer',
    author_email='Harryshomer@gmail.com',
    license='MIT',
    packages=find_packages(),
    install_requires=['BeautifulSoup4', 'requests', 'lxml', 'html5lib', 'pandas', 'sphinx', 'pytest'],
    zip_safe=False
)



""" Tests for 'scrape_functions.py' """

import pandas as pd

from hockey_scraper.nhl import scrape_functions


def test_scrape_list_of_games():
    """ Tests that it correctly scraped a given list of [game_id, date]"""
    games = [
        {'game_id': '2017020001', 'date': '2017-10-04', 'status': 'Final'},
        {'game_id': '2017020746', 'date': '2018-01-25', 'status': 'Final'},
        {'game_id': '2017020450', 'date': '2017-12-09', 'status': 'Final'},
        {'game_id': '2017030311', 'date': '2018-05-11', 'status': 'Final'}
    ]

    # First try without shifts
    pbp, shifts = scrape_functions.scrape_list_of_games(games[:2], False)
    assert isinstance(pbp, pd.DataFrame)
    assert shifts is None
    assert pbp.shape[0] == 614

    # Second we try with shifts
    pbp, shifts = scrape_functions.scrape_list_of_games(games[2:], True)
    assert isinstance(pbp, pd.DataFrame)
    assert isinstance(shifts, pd.DataFrame)
    assert pbp.shape[0] == 602
    assert shifts.shape[0] == 1531

    # Third we feed it a bullshit game_id and see what happens
    pbp, shifts = scrape_functions.scrape_list_of_games([{"game_id": "2016022000", "date": "", "status": ""}], True)
    assert pbp is None
    assert shifts is None
